local L = LibStub("AceLocale-3.0"):GetLocale("ShockAndAwe")
local media = LibStub:GetLibrary("LibSharedMedia-3.0")

local format, len, lower = _G.string.format, _G.string.len, _G.string.lower
local gsub, trim = _G.string.gsub, _G.strtrim

local frame = CreateFrame("Frame", "SAA_ExportFrame", UIParent, "DialogBoxFrame")

local outputText = ""
if not ShockAndAwe.vars then
	ShockAndAwe.vars = {}
end

function ShockAndAwe:ExportToSim()
	if self:CheckIfBuffed() then
		self:PrintMsg(L["warn_sim_export"], ShockAndAwe.db.char.warning.colour, 5)
	end
	outputText = ""
	self:ExportStats()
	self:ExportGlyphs()
	self:ExportTalents()
	StaticPopup_Show("SAA_EXPORT_ENHSIM")
end

StaticPopupDialogs["SAA_EXPORT_ENHSIM"] = {
	text = L["export_enhsim"],
	button1 = ACCEPT,
	button2 = CLOSE,
	hasEditBox = 1,
	OnShow = function()
		local editBox = getglobal(this:GetName().."EditBox")
		editBox:SetText(outputText)
		editBox:HighlightText()
		editBox:SetAutoFocus(false)
		editBox:SetJustifyH("LEFT")
		editBox:SetJustifyV("TOP")
		editBox:SetFocus()
	end,
	EditBoxOnEnterPressed = function()
		this:GetParent():Hide();
	end,
	EditBoxOnEscapePressed = function()
		this:GetParent():Hide();
	end,
	OnHide = function()
		getglobal(this:GetName().."EditBox"):SetText("");
	end,
	timeout = 0,
	hideOnEscape = 1,
}

function ShockAndAwe:AddLine(text)
	if self.db.char.debug then
		self:Print(text)
	end
	if text then
		outputText = outputText.."\r\n"..trim(text)
	end
end

function ShockAndAwe:ExportStats()
	local rating, percentage
	outputText = ""
	self:AddLine("###############################")
	self:AddLine("## ShockAndAwe EnhSim Export ##")
	self:AddLine("###############################")
	local _, race = UnitRace("player")
	self:AddLine("race                 "..lower(race))
	local mh, oh = UnitAttackSpeed("player")
	local haste = GetCombatRatingBonus(18)
	local mhspeed = mh / (1 - haste/100)
	local ohspeed = oh / (1 - haste/100)
	self:AddLine("mh_speed             "..format("%.1f",mhspeed).."0")
	self:AddLine("oh_speed             "..format("%.1f",ohspeed).."0")
	
	self:AddLine("mh_dps               "..self:GetWeaponDPS(16))
	self:AddLine("oh_dps               "..self:GetWeaponDPS(17))
	
	self:AddLine("mh_crit              "..format("%.2f",GetCritChance()))
	self:AddLine("oh_crit              "..format("%.2f",GetCritChance()))
	
	percentage = GetCombatRatingBonus(6) -- Melee Hit rating
	self:AddLine("mh_hit               "..format("%.2f",percentage))
	self:AddLine("oh_hit               "..format("%.2f",percentage))
	
	rating = GetCombatRating(24) -- Expertise rating
	self:AddLine("mh_expertise_rating  "..format("%.2f",rating))
	self:AddLine("oh_expertise_rating  "..format("%.2f",rating))
	
	local ap, buff, debuff = UnitAttackPower("player")
	local power = ap + buff + debuff
	self:AddLine("ap                   "..power)
	self:AddLine("haste                "..format("%.2f",haste))
	
	percentage = GetCombatRatingBonus(25) -- Armour Penetration rating
	self:AddLine("armor_penetration    "..format("%.2f",percentage))
	
	self:AddLine("str                  "..UnitStat("player",1))
	self:AddLine("agi                  "..UnitStat("player",2))
	self:AddLine("int                  "..UnitStat("player",4))
	self:AddLine("spi                  "..UnitStat("player",5))
	
	self:AddLine("spellpower           "..GetSpellBonusDamage(4))
	self:AddLine("spell_crit           "..format("%.2f",GetSpellCritChance(4)))
	
	percentage = GetCombatRatingBonus(8) -- Spell hit rating
--~ 	if UnitFactionGroup("player") == "Alliance" then
--~ 		percentage = percentage + 1
--~ 	end
	self:AddLine("spell_hit            "..format("%.2f",percentage))
	
	self:AddLine("max_mana             "..UnitManaMax("player"))
	local _, manaregen = GetManaRegen()
	self:AddLine("mp5                  "..format("%d", manaregen * 5))
	
	if ShockAndAwe.db.char.binding.mhspell == L["Windfury Weapon"] then
		self:AddLine("mh_imbue             windfury")
	elseif ShockAndAwe.db.char.binding.mhspell == L["Flametongue Weapon"] then
		self:AddLine("mh_imbue             flametongue")
	else
		self:AddLine("mh_imbue             -")
	end
	if ShockAndAwe.db.char.binding.ohspell == L["Windfury Weapon"] then
		self:AddLine("oh_imbue             windfury")
	elseif ShockAndAwe.db.char.binding.ohspell == L["Flametongue Weapon"] then
		self:AddLine("oh_imbue             flametongue")
	else
		self:AddLine("oh_imbue             -")
	end
	self:AddLine(" ")
	self:ExportItemDetails("mh", 16, true)
	self:ExportItemDetails("oh", 17, true)
	self:AddLine(" ")
	
	self:ExportItemDetails("trinket1         ", 13, false)
	self:ExportItemDetails("trinket2         ", 14, false)
	self:AddLine(" ")
	self:ExportItemDetails("totem            ", 18, false)
	self:AddLine("set_bonus            - ## not currently available in the addon export")
	self:AddLine("metagem              "..self:getMetaGemName())
end

function ShockAndAwe:ExportItemDetails(slotname, slotid, weapon)
	local slotLink = GetInventoryItemLink("player", slotid)
	if slotLink then
		local itemName, itemString, _, _, _, _, itemSubType = GetItemInfo(slotLink)
		local _, itemID, enchantID = strsplit(":", itemString)
		if weapon then -- looking up weapon so we want enchant details
			self:AddLine(slotname.."_enchant           "..self:getEnchantName(enchantID))
			if itemSubType == L["One-Handed Axes"] or itemSubType == L["Two-Handed Axes"] then
				self:AddLine(slotname.."_weapon            axe")
			else
				self:AddLine(slotname.."_weapon            -")
			end
		else
			-- regular item either trinket or totem
			self:AddLine(slotname.."    "..self:getSimItemName(itemID))
		end
	else
		self:AddLine(slotname.."    -")
	end
end

function ShockAndAwe:getSimItemName(itemID)
	-- Totems
	if itemID == "33507" then
		return "stonebreakers_totem"
	elseif itemID == "33506" then
		return "skycall_totem"
	elseif itemID == "32330" then
		return "totem_of_ancestral_guidance"
	elseif itemID == "27815" then
		return "totem_of_the_astral_winds"
	elseif itemID == "28248" then
		return "totem_of_the_void"
	elseif itemID == "40267" then
		return "totem_of_hex"
	elseif itemID == "40710" then
		return "totem_of_splintering"
	elseif itemID == "40322" then
		return "totem_of_dueling"
	elseif itemID == "40708" then
		return "totem_of_the_elemental_plane"
	elseif itemID == "42607" then
		return "deadly_gladiators_totem_of_indomitability"
	elseif itemID == "42606" then
		return "hateful_gladiators_totem_of_indomitability"
	elseif itemID == "42593" then
		return "savage_gladiators_totem_of_indomitability"
	elseif itemID == "42602" then
		return "deadly_gladiators_totem_of_survival"
	elseif itemID == "42601" then
		return "hateful_gladiators_totem_of_survival"
	elseif itemID == "42594" then
		return "savage_gladiators_totem_of_survival"
	end
	-- Trinkets
	if itemID == "28830" then
		return "dragonspine_trophy"
	elseif itemID == "32419" then
		return "ashtongue_talisman"
	elseif itemID == "32505" then
		return "madness_of_the_betrayer"
	elseif itemID == "28034" then
		return "hourglass_of_the_unraveller"
	elseif itemID == "30627" then
		return "tsunami_talisman"
	elseif itemID == "34472" then
		return "shard_of_contempt"
	elseif itemID == "33831" then
		return "berserkers_call"
	elseif itemID == "29383" then
		return "bloodlust_brooch"
	elseif itemID == "28288" then
		return "abacus_of_violent_odds"
	elseif itemID == "32658" then
		return "badge_of_tenacity"
	elseif itemID == "35702" then
		return "shadowsong_panther"
	elseif itemID == "34427" then
		return "blackened_naaru_silver"
	elseif itemID == "31856" then
		return "darkmoon_card_crusade"
	elseif itemID == "40431" then
		return "fury_of_the_five_flights"
	elseif itemID == "40256" then
		return "grim_toll"
	elseif itemID == "39257" then
		return "loathebs_shadow"
	elseif itemID == "40684" then
		return "mirror_of_truth"
	elseif itemID == "32483" then
		return "the_skull_of_guldan"
	elseif itemID == "37390" then
		return "meteorite_whetstone"
	elseif itemID == "39229" then
		return "embrace_of_the_spider"
	elseif itemID == "40255" then
		return "dying_curse"
	elseif itemID == "40432" then
		return "illustration_of_the_dragon_soul"
	elseif itemID == "40682" then
		return "sundial_of_the_exiled"
	elseif itemID == "37660" then
		return "forge_ember"
	elseif itemID == "37723" then
		return "incisor_fragment"
	elseif itemID == "37873" then
		return "mark_of_the_war_prisoner"
	elseif itemID == "37166" then
		return "sphere_of_red_dragons_blood"
	elseif itemID == "36972" then
		return "tome_of_arcane_phenomena"
	elseif itemID == "40531" then
		return "mark_of_norgannon"
	elseif itemID == "44014" then
		return "fezziks_pocketwatch"
	elseif itemID == "43836" then
		return "thorny_rose_brooch"
	elseif itemID == "38764" then
		return "rune_of_finite_variation"
	elseif itemID == "40371" then
		return "bandits_insignia"
	elseif itemID == "44253" then
		return "darkmoon_card_greatness"
	elseif itemID == "37264" then
		return "pendulum_of_telluric_currents"
	elseif itemID == "37064" then
		return "vestige_of_haldor"
	elseif itemID == "42395" then
		return "twilight_serpent"
	end
	return "-"
end

function ShockAndAwe:getEnchantName(enchantID)
	local enchantText = "-"
	if enchantID == "2673" then
		enchantText ="mongoose"
	elseif enchantID == "3225" then
		enchantText ="executioner"
	elseif enchantID == "1900" then
		enchantText ="crusader"
	elseif enchantID == "3273" then
		enchantText ="deathfrost"
	elseif enchantID == "3789" then
		enchantText ="berserking"
	else
		enchantText ="-"
	end
	return enchantText
end

function ShockAndAwe:getMetaGemName()
	local metagemText = "-"
	local metagemID = self:GetMetaGemID()
	if metagemID == "32409" then
		metagemText = "relentless_earthstorm_diamond"
	elseif metagemID == "32410" then
		metagemText = "thundering_skyfire_diamond"
	elseif metagemID == "34220" then
		metagemText = "chaotic_skyfire_diamond"
	elseif metagemID == "41285" then
		metagemText = "chaotic_skyflare_diamond"
	elseif metagemID == "41333" then
		metagemText = "ember_skyflare_diamond"
	elseif metagemID == "41398" then
		metagemText = "relentless_earthsiege_diamond"
	else 
		metagemText = "-"
	end
	return metagemText
end

function ShockAndAwe:GetWeaponDPS(slot)
	if self.db.char.debug then
		self:Print("Scanning tooltip for slot "..slot)
	end
	if not self.vars.tooltip then
		self:CreateTooltip()
	end
	self.vars.tooltip:ClearLines()
	self.vars.tooltip:SetInventoryItem("player", slot)
	local dps = nil
	local maxlines = self.vars.tooltip:NumLines()
	if maxlines > 10 then maxlines = 10 end
	for i=1,maxlines do
		if self.db.char.debug then
			self:Print("line "..i.." contains:"..self.vars.Llines[i]:GetText())
		end
		local _,_, dps = string.find(self.vars.Llines[i]:GetText(), L["DPS"])
		if dps and tonumber(dps) then
			return format("%.1f",dps)
		end
	end
	return "0.0 # unable to read weapon dps please check"
end

function ShockAndAwe:CheckIfBuffed()
	local index = 1
	local buffCount = 0
	while UnitBuff("PLAYER", index) do
		local name = UnitBuff("PLAYER", index)
		if name ~= L["Heroic Presence"] then
			buffCount = buffCount + 1
		end
		index = index + 1
	end
	if buffCount > 0 then 
		return true
	else
		return false
	end
end

function ShockAndAwe:ExportGlyphs()
	local numglyphs = GetNumGlyphSockets()
	local major = 0
	local minor = 0
	self:AddLine(" ")
	for index = 1, numglyphs do
		local _, _, spellID = GetGlyphSocketInfo(index)
		if spellID then
			if self.db.char.debug then
				self:Print("found glyph "..index.." spell :"..spellID)
			end
			-- 55447 flame shock
			if spellID == 55448 then
				major = major + 1
				self:AddLine("glyph_major"..major.."         lightning_shield")
			elseif spellID == 55451 then
				major = major + 1
				self:AddLine("glyph_major"..major.."         flametongue_weapon")
			elseif spellID == 55453 then
				major = major + 1
				self:AddLine("glyph_major"..major.."         lightning_bolt")
			elseif spellID == 55445 then
				major = major + 1
				self:AddLine("glyph_major"..major.."         windfury_weapon")
			elseif spellID == 55446 then
				major = major + 1
				self:AddLine("glyph_major"..major.."         stormstrike")
			elseif spellID == 55442 then  
				major = major + 1
				self:AddLine("glyph_major"..major.."         earth_shock")
			elseif spellID == 55454 then
				major = major + 1
				self:AddLine("glyph_major"..major.."         lava")
			elseif spellID == 55444 then
				major = major + 1
				self:AddLine("glyph_major"..major.."         lava_lash")
			elseif spellID == 55436 then
				major = major + 1
				self:AddLine("glyph_major"..major.."         water_mastery")
			end
		end
	end
	for index = major + 1,4 do
		self:AddLine("glyph_major"..index.."         - ## slot empty")
	end
	self:AddLine(" ")
	self:AddLine("glyph_minor1         - ## no useful glyphs current implemented in the sim")
	self:AddLine("glyph_minor2         - ## no useful glyphs current implemented in the sim")
	self:AddLine("glyph_minor3         - ## no useful glyphs current implemented in the sim")
end

function ShockAndAwe:ExportTalents()
	local currRank, maxRank
	self:AddLine(" ")
	self:AddLine(" ")
	self:AddLine("#############")
	self:AddLine("## Talents ##")
	self:AddLine("#############")

	--nameTalent, icon, tier, column, currRank, maxRank= GetTalentInfo(tree,index)
	_, _, _, _, currRank, maxRank = GetTalentInfo(2,3) -- Ancestral Knowledge
	self:AddLine("ancestral_knowledge       "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(2,7) -- Improved Shields
	self:AddLine("improved_shields          "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(2,15) -- Mental Dexterity
	self:AddLine("mental_dexterity          "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(2,9) -- Shamanistic Focus
	self:AddLine("shamanistic_focus         "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(2,11) -- Flurry
	self:AddLine("flurry                    "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(2,8) -- Elemental Weapons
	self:AddLine("elemental_weapons         "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(2,16) -- Unleashed Rage
	self:AddLine("unleashed_rage            "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(2,17) -- Weapon Mastery
	self:AddLine("weapon_mastery            "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(2,18) -- Dual Wield Specialisation
	self:AddLine("dual_wield_specialization "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(2,21) -- Mental Quickness
	self:AddLine("mental_quickness          "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(2,23) -- Imp.Stormstrike
	self:AddLine("improved_stormstrike      "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(2,24) -- Static Shock
	self:AddLine("static_shock              "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(2,27) -- Maelstrom Weapon
	self:AddLine("maelstrom_weapon          "..currRank.."/"..maxRank)
	
	_, _, _, _, currRank, maxRank = GetTalentInfo(1,1) -- Convection
	self:AddLine("convection                "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(1,2) -- Concussion
	self:AddLine("concussion                "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(1,3) -- Call of Flame
	self:AddLine("call_of_flame             "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(1,5) -- Elemental Devastation
	self:AddLine("elemental_devastation     "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(1,6) -- Reverberation
	self:AddLine("reverberation             "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(1,7) -- Elemental Focus
	self:AddLine("elemental_focus           "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(1,8) -- Elemental Fury
	self:AddLine("elemental_fury            "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(1,12) -- Call of Thunder
	self:AddLine("call_of_thunder           "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(1,13) -- Unrelenting Storm
	self:AddLine("unrelenting_storm         "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(1,14) -- Elemental Precision
	self:AddLine("elemental_precision       "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(1,15) -- Lightning Mastery
	self:AddLine("lightning_mastery         "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(1,18) -- Elemental Oath
	self:AddLine("elemental_oath            "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(1,19) -- Lightning Overload
	self:AddLine("lightning_overload        "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(1,22) -- Lava Flows
	self:AddLine("lava_flows                "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(1,23) -- Storm Earth and Fire
	self:AddLine("storm_earth_and_fire      "..currRank.."/"..maxRank)
end

function ShockAndAwe:CheckSetBonus()
	local setnames = { "cyclone_harness", "cataclysm_harness", "desolation_battlegear", "skyshatter_harness", "skyshatter_regalia", "naxx_melee", "gladiators_earthshaker", "naxx_nuker" }
	local slots = { head = 1, shoulder = 3, chest = 5, belt = 6, legs = 7, feet = 8, wrist = 9, gloves = 10 }
	local sets = self:InitialiseSets(setnames)
	
end

function ShockAndAwe:InitialiseSets(setnames)
	local sets = {}
	for _, value in pairs(setnames) do
		sets[value] = {}
		sets[value].count = 0
		sets[value].pieces = 0
		for index = 1, 10 do
			sets[value][index] = {}
			sets[value][index].itemid = 0
		end
	end	
	sets["cyclone_harness"].pieces = 5
	sets["cyclone_harness"].test = 
		function () 
			if sets["cyclone_harness"].count >=2 then
				return "cyclone_harness"
			else
				return "-"
			end
		end
	sets["cyclone_harness"][slots.head].itemid = 29040
	sets["cyclone_harness"][slots.shoulder].itemid = 29043
	sets["cyclone_harness"][slots.chest].itemid = 29038
	sets["cyclone_harness"][slots.legs].itemid = 29042
	sets["cyclone_harness"][slots.gloves].itemid = 29039
	return sets
end

function ShockAndAwe:GetMetaGemID()
	local slotLink = GetInventoryItemLink("player", 1) -- head slot item
	if slotLink then
		for index = 1,3 do
			local _, gemLink = GetItemGem(slotLink,index)
			if gemLink then
				local _, itemString, _, _, _, _, gemType = GetItemInfo(gemLink)
				if(gemType == "Meta") then
					local _, itemID = strsplit(":", itemString)
					return itemID
				end
			end
		end
	end
	return "0"
end

---------------------
-- tooltip scanning
---------------------

function ShockAndAwe:CreateTooltip()
	local tt = CreateFrame("GameTooltip")
	self.vars.tooltip = tt
	tt:SetOwner(UIParent, "ANCHOR_NONE")
	self.vars.Llines, self.vars.Rlines = {}, {}
	for i=1,30 do
		self.vars.Llines[i], self.vars.Rlines[i] = tt:CreateFontString(), tt:CreateFontString()
		self.vars.Llines[i]:SetFontObject(GameFontNormal)
		self.vars.Rlines[i]:SetFontObject(GameFontNormal)
		tt:AddFontStrings(self.vars.Llines[i], self.vars.Rlines[i])
	end
end

